/*
 * Copyright (c) 1998 Peter Bentley (pete@sorted.org)
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL PETER BENTLEY BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * 
 * $Id: proputil.c,v 1.5 1998/06/05 19:21:51 pete Exp $
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <proplist.h>
#include "proputil.h"

void PLDictAdd1( proplist_t dict, char *key, proplist_t value )
{

    proplist_t p = PLMakeString( key );
    PLInsertDictionaryEntry( dict, p, value );
    /* PLRelease( p ); */
}

void PLDictAdd2( proplist_t dict, char *key, char *value )
{

    proplist_t p = PLMakeString( value );
    PLDictAdd1( dict, key, p );
    /* PLRelease( p ); */
}

proplist_t PLGetDict( proplist_t dict, char *key )
{
    proplist_t	k, v;

    if( ! dict )
	return NULL;
    if( ! key )
	return NULL;
    k = PLMakeString( key );
    v = PLGetDictionaryEntry( dict, k );
    PLRelease( k );
    return v;
}

char *PLGetDictString( proplist_t dict, char *key )
{
    proplist_t p = PLGetDict( dict, key );
    if( p )
	return PLGetString( p );
    return NULL;
}

int PLGetDictInt( proplist_t dict, char *key, int dflt )
{
    char *s = PLGetDictString( dict, key );
    if( s )
	return atoi( s );
    else
	return dflt;
}

double PLGetDictDouble( proplist_t dict, char *key, double dflt )
{
    char *s = PLGetDictString( dict, key );
    if( s )
	return atof( s );
    else
	return dflt;
}


char *PLGetArrayString( proplist_t array, int num )
{
    proplist_t p = PLGetArrayElement( array, num );
    if( p )
	return PLGetString( p );
    return NULL;
}

int PLOrderCompare( const proplist_t p1, const proplist_t p2 )
{
    int o1 = PLGetDictInt( p1, "Order", 0 );
    int o2 = PLGetDictInt( p2, "Order", 0 );

    if( o1 > o2 )
	return 1;
    if( o2 > o1 )
	return -1;
    return 0;
}

int PLSaveWithBackup( proplist_t props )
{
    char filename[ 255 ], backname[ 255 ];		/* XXX */

    strcpy( filename, PLGetString( PLGetFilename( props )));
    strcpy( backname, filename );
    strcat( backname, ".bak" );
    /* printf( "Save to %s, backup to %s\n", filename, backname ); */
    if( rename( filename, backname ))
    {
	fprintf( stderr, "Backup %s to %s failed\n", filename, backname );
	return NO;
    }
    return PLSave( props, YES );
}
